import numpy as np
import pandas as pd

#%% Make continous data series

#Group values to 1 min values. Make sure that the true and false value flags still is correct. Forward-fill relevant data and interpolate the missing values 
#within 5 min timestep. Set data limitation according to information. 
# Consider to take 5 min timestep for interpolation out as well as 1 min grouping. 
#
def make_continous_series(raw_data, start, end):
    #drop if nan value present
    #raw_data=raw_data.dropna(subset = ['value_no_errors'])
    
 #Group to mean values in 1 minute scale
    timeseries = raw_data.set_index('time').resample('T').mean()
    #To explore statistics
    ts_meanminmax = raw_data.set_index('time').resample('T').agg({'value_no_errors': ['mean', 'min', 'max'], 'outbound':['min','max']}) 
    #ensures that there is true-false values in flags. If value present in "value_no_errors" then true-false value should be false. 
    lcolumn = [col for col in list(timeseries.columns) if col not in ['raw_value', 'value_no_errors']]
    for col in lcolumn: 
        timeseries[col] = timeseries[col].where(timeseries[col]==1, 0)
        timeseries[col] = timeseries[col].astype(bool)
   
    #Forward-fill for datapoints that needs to be forward-filled, and interpolate with rest. 
    temp = pd.Series(timeseries['value_no_errors'])
    temp_pad = temp.interpolate(method='pad')
    temp2 = temp.copy() 
    temp2.loc[timeseries['ffill'] == True] = temp_pad
           
    temp3 = temp2.interpolate(method='linear', limit = 5)
    
    timeseries_interpolated = pd.DataFrame(temp3)
    timeseries_interpolated.columns = ['depth_s']    
    result = pd.concat([timeseries, timeseries_interpolated], axis=1, sort=False)
    result.reset_index(inplace = True)    
        
    #Make continous timeseries with values and use only valid dates
    date_start = pd.to_datetime(start, format = "%d-%m-%Y")
    date_end = pd.to_datetime(end, format = "%d-%m-%Y")
        
    data_lim = result[(result['time'] >= date_start) & (result['time'] <= date_end)]
    #data_lim = result.copy() #JUST FOR TESTING
  
    return data_lim, ts_meanminmax


### Adjust the to a level from the off-set in sensor value if level or position as type. 
def procces_data(value, obs_input, filestring, sourcestring, part=0): 
    j = int(np.where((obs_input['IdMeasurement']==filestring) & (obs_input['Source']==sourcestring)& (obs_input['Version']==part))[0])

    if obs_input.at[j,'Type'] == 'Level' or obs_input.at[j,'Type'] == 'Position':
        value = value+(obs_input.at[j,'zeropoint'])
    else:
        value = value
    return value