
import numpy as np
import pandas as pd
from scipy.signal import find_peaks

#%% DQC functions
# All of the functions are meant for data formatted as Pandas' Series

#Check if the data is stamped errorous
#qualityerror is given value, which is an error-measurement
# Gives True value if value is None
def Stamped_error(values, quality, qualityerror):
    flag_stamp = pd.Series(np.zeros(len(values)),dtype=bool)
    flag_stamp.loc[(quality == qualityerror)]= True
    
    return flag_stamp

#Flag out manually removed periods from table. 
def Remove_man(data, man_remove, name):
    listremove = list(np.where(man_remove['IdMeasurement']==name))
    man_remove1 = man_remove.drop(man_remove[man_remove.IdMeasurement !=name].index).reset_index(inplace=False)
    
    flag_removeman = pd.Series(np.zeros(len(data.value)),dtype=bool)
    if np.asarray(listremove).size != 0:
        for n in range(0,len(man_remove1)):
            start= pd.to_datetime(man_remove1.at[n,'StartTime'], format = "%d-%m-%Y %H:%M")
            end = pd.to_datetime(man_remove1.at[n,'EndTime'], format = "%d-%m-%Y %H:%M")
           
            #Ensures that data is present for this file
            if sum(data['time']>=start) > 0:
                startid = data.loc[data['time']>=start].first_valid_index()
                if sum(data['time']>=end) >0:
                    endid = data.loc[data['time']>=end].first_valid_index()
                else: 
                    endid = len(data)
                flag_removeman.iloc[startid:endid] = True
            elif sum(data['time']>=end) >0:
                startid = 0
                endid = data.loc[data['time']>=end].first_valid_index()
                
                flag_removeman.iloc[startid:endid] = True
            else: 
                pass
                
    return flag_removeman


# Set physically realistic upper and lower values that the data is not allowed to exceed
# Any value outside the realistic range is flagged.
def Out_of_bounds(values,lower,upper):
    flag_outbound = pd.Series(np.zeros(len(values)),dtype=bool)
    flag_outbound.loc[values < lower] = True
    flag_outbound.loc[values > upper] = True
    return(flag_outbound)

# Check if the sensor has frozen and only delivers constant values.
# The user provides a time window that specifies the maximum number of neighboring data points that are allowed to be identical, before it seems suspicious.
# "window_size" of 2 means that the two points before and after a data point (5 in total) are checked if they are identical/frozen.
def Frozen_sensor(values, times ,thres_value, window_size): 
    #values=data.value.copy()
    #times=data.time.copy()
    flag_frozen = pd.Series(np.zeros(len(values)),dtype=bool)
    values_diff = values.diff()
    values_diffminus = values.diff(-1)
    
    values_mean = round(abs(values_diff).rolling(window_size-1).mean(),4)
    values_diffmean = round(abs(values_diffminus).rolling(window_size-1).mean(),4)
    
    flag_frozen.loc[(values_mean == 0) & (values_diff == 0) & (values > thres_value)] = True
    flag_frozen.loc[(values_diffmean == 0) & (values_diffminus == 0) & (values > thres_value)] = True
    
    
    #ensures that the period is above window_size
    flag_frozen_diff = flag_frozen.astype(int).diff()
    s = flag_frozen_diff[flag_frozen_diff ==1].reset_index()
    e = flag_frozen_diff[flag_frozen_diff ==-1].reset_index()

    if len(s) != len(e):
        e = e.append({'index':(len(flag_frozen_diff)-1)}, ignore_index=True)

    timediff = pd.Series(np.zeros(len(s)))
    for t in range(0,len(s)):
        timediff[t] = times[e['index'].iloc[t]]- times[s['index'].iloc[t]]

    
    flag_frozen2 = pd.Series(np.zeros(len(values)),dtype=bool)
    for t in range(0,len(s)):
        if timediff[t] >  pd.Timedelta(minutes=window_size):
            flag_frozen2[int(s['index'].iloc[t]):int(e['index'].iloc[t])] = True
   
    return flag_frozen2

# Check for outliers by looking if there is a sudden increase and drop in values
def Outlier_point(values, threshold1, width1):
    #Flag if there is a peak value of threshold (height) and width (0.2 and 1 respectively)
    flag_outlier = pd.Series(np.zeros(len(values)),dtype=bool)
    peaks, _ = find_peaks(values, threshold=threshold1, width=width1)
    flag_outlier[peaks] = True
  
    return(flag_outlier)
